@@ -1,3 +1,3 @@
-default_app_config = 'security.backends.testing.app.SecurityTestingBackend'
+from .writer import capture_security_logs  # noqa: F401
 
-from .writer import capture_security_logs
+default_app_config = 'security.backends.testing.app.SecurityTestingBackend'
